#!/bin/bash

#################################
#      ISPProtect BanDaemon     #
#     Modern Systemd Installer  #
#################################

# Colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
BLUE='\033[0;34m'
BOLD='\033[1m'
NC='\033[0m'

# Installation paths
INSTALL_DIR="/opt/ispprotect_bandaemon"
SERVICE_NAME="ispprotect_bandaemon"
SERVICE_FILE="/etc/systemd/system/${SERVICE_NAME}.service"

# Installation type flags
UPDATE_INSTALL=false
MIGRATION_INSTALL=false
SERVICE_WAS_RUNNING=false
BACKUP_DIR=""

print_banner() {
    echo ""
    echo -e "${BLUE}${BOLD}╔══════════════════════════════════════╗${NC}"
    echo -e "${BLUE}${BOLD}║        ISPProtect BanDaemon          ║${NC}"
    echo -e "${BLUE}${BOLD}║      Modern Systemd Installer        ║${NC}"
    echo -e "${BLUE}${BOLD}╚══════════════════════════════════════╝${NC}"
    echo ""
}

check_requirements() {
    echo -e "${BLUE}Checking system requirements...${NC}"
    
    # Check if running as root
    if [ "$(id -u)" != "0" ]; then
        echo -e "${RED}✗ This script must be run as root${NC}"
        exit 1
    fi
    
    # Check PHP
    PHP=$(which php)
    if [ -z "${PHP}" ] || [ ! -e "${PHP}" ]; then
        echo -e "${RED}✗ PHP binary not found${NC}"
        exit 1
    fi
    echo -e "${GREEN}✓ PHP found: ${PHP}${NC}"
    
    # Check systemd
    if [ ! -e "/bin/systemctl" ] && [ ! -e "/usr/bin/systemctl" ]; then
        echo -e "${RED}✗ systemctl not found - this installer requires systemd${NC}"
        echo -e "${YELLOW}Use install_initd.sh for traditional init.d systems${NC}"
        exit 1
    fi
    echo -e "${GREEN}✓ systemd found${NC}"
    
    # Check MySQL client
    if ! command -v mysql >/dev/null 2>&1; then
        echo -e "${YELLOW}⚠ MySQL client not found - database setup will be skipped${NC}"
    else
        echo -e "${GREEN}✓ MySQL client found${NC}"
    fi
    
    echo ""
}

install_files() {
    echo -e "${BLUE}Installing application files...${NC}"
    
    # Create installation directory
    mkdir -p "${INSTALL_DIR}"
    
    # Copy main bash script
    if [ -f "ispp_bandaemon" ]; then
        cp "ispp_bandaemon" "${INSTALL_DIR}/"
        chmod 755 "${INSTALL_DIR}/ispp_bandaemon"
        echo -e "${GREEN}✓ Installed ispp_bandaemon script${NC}"
    else
        echo -e "${RED}✗ ispp_bandaemon script not found${NC}"
        exit 1
    fi
    
    # Copy main PHP files
    for file in ispp_bandaemon.php ispp_bd_helper.php ispp_bd_report.php ispp_bd_postlog.php ispp_bd_abusemail.php; do
        if [ -f "$file" ]; then
            cp "$file" "${INSTALL_DIR}/"
            echo -e "${GREEN}✓ Installed $file${NC}"
        else
            echo -e "${YELLOW}⚠ $file not found, skipping${NC}"
        fi
    done
    
    # Copy SourceGuardian loader helper (unencoded)
    if [ -f "ispp_loader.php" ]; then
        cp "ispp_loader.php" "${INSTALL_DIR}/"
        echo -e "${GREEN}✓ Installed SourceGuardian loader helper${NC}"
    else
        echo -e "${YELLOW}⚠ ispp_loader.php not found, skipping${NC}"
    fi
    
    # Copy directories
    for dir in conf.d tpl includes; do
        if [ -d "$dir" ]; then
            cp -r "$dir" "${INSTALL_DIR}/"
            echo -e "${GREEN}✓ Installed $dir/${NC}"
        else
            echo -e "${YELLOW}⚠ $dir not found, skipping${NC}"
        fi
    done
    
    # Note: License will be handled separately in setup_license() function
    
    # Install postlog script in PHP directories
    for php_dir in "/usr/share/php" "/usr/share/php5" "/usr/share/php7.0" "/usr/share/php7.4" "/usr/share/php8.0" "/usr/share/php8.1"; do
        if [ -d "$php_dir" ] && [ -f "ispp_bd_postlog.php" ]; then
            cp ispp_bd_postlog.php "$php_dir/"
            chmod 755 "$php_dir/ispp_bd_postlog.php"
            echo -e "${GREEN}✓ Installed postlog script in $php_dir${NC}"
        fi
    done
    
    # Set permissions
    chown -R root:root "${INSTALL_DIR}"
    chmod -R 640 "${INSTALL_DIR}"
    find "${INSTALL_DIR}" -type d -exec chmod 750 {} \;
    chmod 644 "${INSTALL_DIR}"/ispp_*.php
    chmod 755 "${INSTALL_DIR}/ispp_bandaemon"
    
    echo ""
}

create_system_symlink() {
    echo -e "${BLUE}Creating system symlink...${NC}"
    
    # Create symlink in /usr/local/bin for system-wide access
    SYMLINK_PATH="/usr/local/bin/ispp_bandaemon"
    
    # Remove existing symlink if it exists
    if [ -L "${SYMLINK_PATH}" ]; then
        rm -f "${SYMLINK_PATH}"
        echo -e "${YELLOW}⚠ Removed existing symlink${NC}"
    elif [ -f "${SYMLINK_PATH}" ]; then
        echo -e "${RED}✗ File exists at ${SYMLINK_PATH} (not a symlink)${NC}"
        echo -e "${YELLOW}Please remove it manually and run the installer again${NC}"
        exit 1
    fi
    
    # Create the symlink
    if ln -s "${INSTALL_DIR}/ispp_bandaemon" "${SYMLINK_PATH}"; then
        echo -e "${GREEN}✓ Created symlink: ${SYMLINK_PATH} → ${INSTALL_DIR}/ispp_bandaemon${NC}"
    else
        echo -e "${RED}✗ Failed to create symlink${NC}"
        exit 1
    fi
    
    # Verify symlink works
    if [ -L "${SYMLINK_PATH}" ] && [ -x "${INSTALL_DIR}/ispp_bandaemon" ]; then
        echo -e "${GREEN}✓ Symlink is functional${NC}"
    else
        echo -e "${RED}✗ Symlink verification failed${NC}"
        echo -e "${YELLOW}  Symlink exists: $([ -L "${SYMLINK_PATH}" ] && echo "Yes" || echo "No")${NC}"
        echo -e "${YELLOW}  Target executable: $([ -x "${INSTALL_DIR}/ispp_bandaemon" ] && echo "Yes" || echo "No")${NC}"
        exit 1
    fi
    
    echo ""
}

create_systemd_service() {
    echo -e "${BLUE}Installing systemd service...${NC}"
    
    # Copy systemd service file
    if [ -f "etc/systemd/system/ispprotect_bandaemon.service" ]; then
        cp "etc/systemd/system/ispprotect_bandaemon.service" "${SERVICE_FILE}"
        echo -e "${GREEN}✓ Systemd service file copied${NC}"
    else
        echo -e "${RED}✗ Systemd service file not found: etc/systemd/system/ispprotect_bandaemon.service${NC}"
        exit 1
    fi
    
    # Verify installation
    if [ -f "${SERVICE_FILE}" ]; then
        echo -e "${GREEN}✓ Systemd service installed${NC}"
    else
        echo -e "${RED}✗ Failed to install systemd service${NC}"
        exit 1
    fi
    
    # Reload systemd
    systemctl daemon-reload
    echo -e "${GREEN}✓ Systemd configuration reloaded${NC}"
    echo ""
}

handle_existing_installation() {
    echo -e "${BLUE}Checking for previous installation...${NC}"
    
    # Check for current ISPProtect installation (update scenario)
    if [ -d "$INSTALL_DIR" ] && [ -f "${INSTALL_DIR}/config.inc.php" ]; then
        echo -e "${YELLOW}Found existing ISPProtect BanDaemon installation${NC}"
        echo -e "${BLUE}Performing update...${NC}"
        
        # Stop current service and track if it was running
        SERVICE_WAS_RUNNING=false
        if systemctl is-active "$SERVICE_NAME" >/dev/null 2>&1; then
            SERVICE_WAS_RUNNING=true
            echo -e "${YELLOW}Stopping current service...${NC}"
            systemctl stop "$SERVICE_NAME" 2>/dev/null || true
            echo -e "${GREEN}✓ Service stopped${NC}"
        fi
        
        # Clean up old PID files (migration from /var/run to /run)
        for old_pid in "/var/run/ispprotect_bandaemon.pid" "/var/run/ispconfig3_bandaemon.pid"; do
            if [ -f "$old_pid" ]; then
                rm -f "$old_pid"
                echo -e "${GREEN}✓ Removed old PID file: $old_pid${NC}"
            fi
        done
        
        # Backup configuration files
        BACKUP_DIR="/tmp/ispprotect_backup_$(date +%Y%m%d_%H%M%S)"
        mkdir -p "$BACKUP_DIR"
        # Secure backup directory - only root can read (contains sensitive credentials)
        chmod 700 "$BACKUP_DIR"
        chown root:root "$BACKUP_DIR" 2>/dev/null || true
        
        for file in config.inc.php license.key conf.d evidence; do
            if [ -e "${INSTALL_DIR}/$file" ]; then
                cp -a "${INSTALL_DIR}/$file" "$BACKUP_DIR/"
                echo -e "${GREEN}✓ Backed up $file to $BACKUP_DIR${NC}"
            fi
        done
        echo -e "${YELLOW}  (Backup secured with 700 permissions)${NC}"
        
        # Mark as update installation
        UPDATE_INSTALL=true
        echo -e "${GREEN}✓ Ready for update installation${NC}"
    fi
    
    # Check for old ISPConfig3 installation (migration scenario)
    if [ -d "/opt/ispconfig3_bandaemon" ]; then
        echo -e "${YELLOW}Found old ISPConfig3 installation${NC}"
        echo -e "${YELLOW}Migrating configuration...${NC}"
        
        # Stop old service
        for old_service in ispconfig3_bandaemon ispprotect_bandaemon; do
            if [ -f "/etc/init.d/$old_service" ]; then
                service "$old_service" stop 2>/dev/null || true
                # Disable old init scripts
                if command -v update-rc.d >/dev/null 2>&1; then
                    update-rc.d "$old_service" disable 2>/dev/null || true
                elif command -v chkconfig >/dev/null 2>&1; then
                    chkconfig "$old_service" off 2>/dev/null || true
                fi
            fi
        done
        
        # Migrate configuration files
        if [ -f "/opt/ispconfig3_bandaemon/config.inc.php" ]; then
            mkdir -p "${INSTALL_DIR}"
            cp "/opt/ispconfig3_bandaemon/config.inc.php" "${INSTALL_DIR}/"
            echo -e "${GREEN}✓ Configuration migrated${NC}"
        fi
        
        # Migrate other files
        for file in conf.d license.key; do
            if [ -e "/opt/ispconfig3_bandaemon/$file" ]; then
                mkdir -p "${INSTALL_DIR}"
                cp -a "/opt/ispconfig3_bandaemon/$file" "${INSTALL_DIR}/"
                echo -e "${GREEN}✓ $file migrated${NC}"
            fi
        done
        
        echo -e "${YELLOW}⚠ Old installation at /opt/ispconfig3_bandaemon can be removed manually${NC}"
        MIGRATION_INSTALL=true
    fi
    
    echo ""
}

setup_database() {
    echo -e "${BLUE}Database setup...${NC}"
    
    if ! command -v mysql >/dev/null 2>&1; then
        echo -e "${YELLOW}MySQL client not available, skipping database setup${NC}"
        return
    fi
    
    # Check if database already exists
    DB_EXISTS=false
    USER_EXISTS=false
    
    # Try to detect existing database
    if [ -f "/etc/mysql/debian.cnf" ]; then
        DB_CHECK=$(mysql --defaults-file=/etc/mysql/debian.cnf -e "SELECT COUNT(*) FROM INFORMATION_SCHEMA.SCHEMATA WHERE SCHEMA_NAME='ispprotect_bandaemon'" 2>/dev/null | tail -n 1 | tr -d '\n\r ' || echo "0")
        USER_CHECK=$(mysql --defaults-file=/etc/mysql/debian.cnf -e "SELECT COUNT(*) FROM mysql.user WHERE User='ispp_bandaemon'" 2>/dev/null | tail -n 1 | tr -d '\n\r ' || echo "0")
    else
        echo -e "${YELLOW}Checking existing database (MySQL root password required)...${NC}"
        DB_CHECK=$(mysql -u root -p -e "SELECT COUNT(*) FROM INFORMATION_SCHEMA.SCHEMATA WHERE SCHEMA_NAME='ispprotect_bandaemon'" 2>/dev/null | tail -n 1 | tr -d '\n\r ' || echo "0")
        USER_CHECK=$(mysql -u root -p -e "SELECT COUNT(*) FROM mysql.user WHERE User='ispp_bandaemon'" 2>/dev/null | tail -n 1 | tr -d '\n\r ' || echo "0")
    fi
    
    # Ensure variables are clean integers
    DB_CHECK=$(echo "$DB_CHECK" | grep -o '^[0-9]*$' || echo "0")
    USER_CHECK=$(echo "$USER_CHECK" | grep -o '^[0-9]*$' || echo "0")
    
    if [ "$DB_CHECK" -gt "0" ]; then
        DB_EXISTS=true
        echo -e "${GREEN}✓ Database 'ispprotect_bandaemon' already exists${NC}"
    fi
    
    if [ "$USER_CHECK" -gt "0" ]; then
        USER_EXISTS=true
        echo -e "${GREEN}✓ Database user 'ispp_bandaemon' already exists${NC}"
    fi
    
    # Handle different scenarios
    if [ "${UPDATE_INSTALL:-}" = "true" ] && [ "$DB_EXISTS" = "true" ] && [ "$USER_EXISTS" = "true" ]; then
        echo -e "${GREEN}✓ Using existing database configuration${NC}"
        echo -e "${YELLOW}⚠ Database password will be preserved from existing configuration${NC}"
        PWORD=""  # Will use existing password from config
    elif [ "$DB_EXISTS" = "true" ] || [ "$USER_EXISTS" = "true" ]; then
        echo -e "${YELLOW}⚠ Database or user partially exists${NC}"
        printf "Recreate database and user? [y/N]: "
        read -r RECREATE
        if [ "$RECREATE" = "Y" ] || [ "$RECREATE" = "y" ]; then
            create_database_and_user
        else
            echo -e "${YELLOW}Skipping database setup - using existing${NC}"
            PWORD=""
        fi
    else
        printf "Create database and user? [Y/n]: "
        read -r CREAT
        if [ "$CREAT" = "" ] || [ "$CREAT" = "Y" ] || [ "$CREAT" = "y" ]; then
            create_database_and_user
        else
            echo -e "${YELLOW}Skipping database setup${NC}"
            PWORD=""
        fi
    fi
    
    echo ""
}

create_database_and_user() {
    # Generate password
    PWORD=$(date +%N%s | sha256sum | head -c 32)
    
    echo -e "${YELLOW}Creating database and user...${NC}"
    
    # Try different MySQL authentication methods
    if [ -f "/etc/mysql/debian.cnf" ]; then
        mysql --defaults-file=/etc/mysql/debian.cnf -e "DROP DATABASE IF EXISTS ispprotect_bandaemon" 2>/dev/null
        mysql --defaults-file=/etc/mysql/debian.cnf -e "DROP USER IF EXISTS 'ispp_bandaemon'@'localhost'" 2>/dev/null
        mysql --defaults-file=/etc/mysql/debian.cnf -e "CREATE DATABASE ispprotect_bandaemon" 2>/dev/null
        mysql --defaults-file=/etc/mysql/debian.cnf -e "CREATE USER 'ispp_bandaemon'@'localhost' IDENTIFIED BY '${PWORD}'" 2>/dev/null
        mysql --defaults-file=/etc/mysql/debian.cnf -e "GRANT ALL PRIVILEGES ON ispprotect_bandaemon.* TO 'ispp_bandaemon'@'localhost'" 2>/dev/null
        mysql --defaults-file=/etc/mysql/debian.cnf -e "FLUSH PRIVILEGES" 2>/dev/null
    else
        mysql -u root -p -e "DROP DATABASE IF EXISTS ispprotect_bandaemon"
        mysql -u root -p -e "DROP USER IF EXISTS 'ispp_bandaemon'@'localhost'"
        mysql -u root -p -e "CREATE DATABASE ispprotect_bandaemon"
        mysql -u root -p -e "CREATE USER 'ispp_bandaemon'@'localhost' IDENTIFIED BY '${PWORD}'"
        mysql -u root -p -e "GRANT ALL PRIVILEGES ON ispprotect_bandaemon.* TO 'ispp_bandaemon'@'localhost'"
        mysql -u root -p -e "FLUSH PRIVILEGES"
    fi
    
    if [ $? -eq 0 ]; then
        echo -e "${GREEN}✓ Database and user created successfully${NC}"
        echo -e "${GREEN}✓ Generated new database password${NC}"
    else
        echo -e "${RED}✗ Database creation failed${NC}"
        PWORD=""
    fi
}

create_configuration() {
    echo -e "${BLUE}Creating configuration...${NC}"
    
    # Generate server identifier
    HASH=$(hostname -f)
    case "$HASH" in
        *.*)
            echo -e "${GREEN}Using hostname: $HASH${NC}"
            ;;
        *)
            HASH=$(hostname)
            case "$HASH" in
                *.*)
                    echo -e "${GREEN}Using hostname: $HASH${NC}"
                    ;;
                *)
                    HASH=$(date +%N%s | sha256sum | head -c 32)
                    echo -e "${YELLOW}Generated server ID: $HASH${NC}"
                    ;;
            esac
            ;;
    esac
    
    # Detect web root
    WWWPATH="/var/www"
    if [ ! -d "$WWWPATH" ]; then
        if [ -d "/srv/www" ]; then
            WWWPATH="/srv/www"
        else
            WWWPATH=""
            echo -e "${YELLOW}⚠ No web root found, WWW protection disabled${NC}"
        fi
    fi
    
    # Handle configuration for updates vs fresh installs
    if [ "${UPDATE_INSTALL:-}" = "true" ]; then
        # For updates, restore backed up config and create new sample
        if [ -n "${BACKUP_DIR:-}" ] && [ -f "$BACKUP_DIR/config.inc.php" ]; then
            cp "$BACKUP_DIR/config.inc.php" "${INSTALL_DIR}/"
            echo -e "${GREEN}✓ Configuration restored from backup${NC}"
        fi
        CONFIG_FILE="${INSTALL_DIR}/config.inc.php.new"
        echo -e "${YELLOW}Creating new configuration template${NC}"
    elif [ -f "${INSTALL_DIR}/config.inc.php" ]; then
        echo -e "${YELLOW}Configuration already exists, creating sample only${NC}"
        CONFIG_FILE="${INSTALL_DIR}/config.inc.php.sample"
    else
        CONFIG_FILE="${INSTALL_DIR}/config.inc.php"
    fi
    
    # Create configuration
    cat > "$CONFIG_FILE" << EOF
<?php
/** ISPProtect BanDaemon Configuration **/

// Database settings
DEFINE('DB_HOST', 'localhost');
DEFINE('DB_NAME', 'ispprotect_bandaemon');
DEFINE('DB_USER', 'ispp_bandaemon');
DEFINE('DB_PASS', '${PWORD}');

// Iptables settings
define('BANDAEMON_TABLE', 'PREROUTING');
define('BANDAEMON_MODE', 'INSERT');

// Logging settings
define('BANDAEMON_SYSLOG', false);
define('BANDAEMON_VERBOSE', false);
define('BANDAEMON_LOGNAME', false);

// Web root path
define('BANDAEMON_WWW_ROOT', '${WWWPATH}');

// Service configuration
define('BANDAEMON_DISABLE_WATCH', '');
define('BANDAEMON_ENABLE_WATCH', '');
define('BANDAEMON_TESTMODE', false);

// Security features
define('BANDAEMON_ABUSE_ADDRESS', false);
define('BANDAEMON_EVIDENCE', false);
define('BANDAEMON_EVIDENCE_SCORE', 100);

// Reporting
define('BANDAEMON_DAILY_REPORT', false);

// Server identification
define('SERVER_IDENT', '${HASH}');

// Logging
define('LOG_FLOOD_ATTEMPTS', true);

// IP whitelisting
define('WHITELISTED_IPS', '');

// Email settings (for abuse reporting)
define('EMAIL_SENDER', '');
define('EMAIL_SENDER_NAME', 'ISPProtect Security Team');

// SMTP Configuration (optional)
//define('EMAIL_SMTP_HOST', 'localhost');
//define('EMAIL_SMTP_PORT', 25);
//define('EMAIL_SMTP_USER', '');
//define('EMAIL_SMTP_PASS', '');
//define('EMAIL_SMTP_ENCRYPTION', false); // false, 'tls', or 'ssl'
//define('EMAIL_SMTP_AUTH', false);

?>
EOF
    
    echo -e "${GREEN}✓ Configuration created: $CONFIG_FILE${NC}"
    
    # Restore other backed up files during update
    if [ "${UPDATE_INSTALL:-}" = "true" ] && [ -n "${BACKUP_DIR:-}" ]; then
        for file in license.key conf.d evidence; do
            if [ -e "$BACKUP_DIR/$file" ] && [ ! -e "${INSTALL_DIR}/$file" ]; then
                cp -a "$BACKUP_DIR/$file" "${INSTALL_DIR}/"
                echo -e "${GREEN}✓ Restored $file from backup${NC}"
            fi
        done
    fi
    
    echo ""
}

setup_license() {
    echo -e "${BLUE}License setup...${NC}"
    
    # Skip license setup for updates and migrations - license should be preserved
    if [ "${UPDATE_INSTALL:-}" = "true" ] || [ "${MIGRATION_INSTALL:-}" = "true" ]; then
        if [ -f "${INSTALL_DIR}/license.key" ]; then
            echo -e "${GREEN}✓ Existing license preserved${NC}"
        else
            echo -e "${YELLOW}⚠ No license found in existing installation${NC}"
        fi
        echo ""
        return
    fi
    
    # For fresh installations, check if license already exists (shouldn't happen, but safety check)
    if [ -f "${INSTALL_DIR}/license.key" ]; then
        echo -e "${GREEN}✓ License file already exists${NC}"
        echo ""
        return
    fi
    
    echo ""
    echo -e "${BLUE}${BOLD}ISPProtect BanDaemon License Setup${NC}"
    echo ""
    echo "Please choose one of the following options:"
    echo ""
    echo "1. Enter your BanDaemon or ISPProtect 12 month scanner license key"
    echo "2. Start a 14-day free trial (press Enter)"
    echo ""
    printf "Enter your license key (or press Enter for trial): "
    read -r LICENSE_KEY
    
    if [ -z "$LICENSE_KEY" ]; then
        # User pressed Enter - create trial license
        echo -e "${YELLOW}Creating 14-day trial license...${NC}"
        echo "TRIAL" > "${INSTALL_DIR}/license.key"
        echo -e "${GREEN}✓ Trial license created${NC}"
        echo -e "${YELLOW}⚠ This is a 14-day trial license. Permanent license keys can be purchased at https://ispprotect.com${NC}"
    else
        # User entered a license key
        echo -e "${BLUE}Installing license.key file...${NC}"
        echo "$LICENSE_KEY" > "${INSTALL_DIR}/license.key"
        echo -e "${GREEN}✓ License key installed${NC}"
    fi
    
    # Set proper permissions on license file
    chmod 640 "${INSTALL_DIR}/license.key"
    chown root:root "${INSTALL_DIR}/license.key"
    
    echo ""
}

setup_logrotate() {
    echo -e "${BLUE}Setting up log rotation...${NC}"
    
    cat > /etc/logrotate.d/ispprotect_bandaemon << 'EOF'
/var/log/ispprotect_bandaemon.log {
    daily
    rotate 7
    missingok
    create 640 root root
    compress
    notifempty
    postrotate
        /bin/systemctl reload-or-restart ispprotect_bandaemon.service > /dev/null 2>&1 || true
    endscript
}
EOF
    
    echo -e "${GREEN}✓ Log rotation configured${NC}"
    echo ""
}

setup_cron() {
    printf "Setup daily report cron job? [y/N]: "
    read -r SETUP_CRON
    
    if [ "$SETUP_CRON" = "Y" ] || [ "$SETUP_CRON" = "y" ]; then
        echo -e "${BLUE}Setting up cron job...${NC}"
        
        # Remove old cron entries
        rm -f /etc/cron.d/ispprotect_daemon /etc/cron.d/ispconfig3_bandaemon
        
        # Create new cron job
        cat > /etc/cron.d/ispprotect_bandaemon << EOF
# ISPProtect BanDaemon daily report
15 0 * * * root ${INSTALL_DIR}/ispp_bd_report.php > /dev/null 2>&1
EOF
        
        echo -e "${GREEN}✓ Daily report cron job created${NC}"
        echo -e "${YELLOW}Don't forget to set BANDAEMON_DAILY_REPORT email in config${NC}"
    fi
    
    echo ""
}

handle_service_startup() {
    echo -e "${BLUE}Service startup...${NC}"
    
    if [ "${UPDATE_INSTALL:-}" = "true" ]; then
        # For updates: restart automatically if it was running before
        if [ "${SERVICE_WAS_RUNNING:-}" = "true" ]; then
            echo -e "${YELLOW}Restarting service (was running before update)...${NC}"
            if systemctl start "$SERVICE_NAME"; then
                echo -e "${GREEN}✓ Service restarted successfully${NC}"
                # Show status
                sleep 2
                if systemctl is-active --quiet "$SERVICE_NAME"; then
                    echo -e "${GREEN}✓ Service is running and active${NC}"
                else
                    echo -e "${YELLOW}⚠ Service started but may need configuration review${NC}"
                fi
            else
                echo -e "${RED}✗ Failed to restart service${NC}"
                echo -e "${YELLOW}Please check configuration and start manually: systemctl start $SERVICE_NAME${NC}"
            fi
        else
            echo -e "${YELLOW}Service was not running before update - not starting automatically${NC}"
        fi
    elif [ "${MIGRATION_INSTALL:-}" = "true" ]; then
        # For migrations: ask if user wants to start
        printf "Start the service now? [Y/n]: "
        read -r START_SERVICE
        if [ "$START_SERVICE" = "" ] || [ "$START_SERVICE" = "Y" ] || [ "$START_SERVICE" = "y" ]; then
            echo -e "${YELLOW}Starting service...${NC}"
            if systemctl start "$SERVICE_NAME"; then
                echo -e "${GREEN}✓ Service started successfully${NC}"
                sleep 2
                if systemctl is-active --quiet "$SERVICE_NAME"; then
                    echo -e "${GREEN}✓ Service is running and active${NC}"
                else
                    echo -e "${YELLOW}⚠ Service started but may need configuration review${NC}"
                fi
            else
                echo -e "${RED}✗ Failed to start service${NC}"
                echo -e "${YELLOW}Please check configuration and start manually: systemctl start $SERVICE_NAME${NC}"
            fi
        fi
    else
        # For fresh installs: ask if user wants to start
        printf "Start the service now? [Y/n]: "
        read -r START_SERVICE
        if [ "$START_SERVICE" = "" ] || [ "$START_SERVICE" = "Y" ] || [ "$START_SERVICE" = "y" ]; then
            echo -e "${YELLOW}Starting service...${NC}"
            if systemctl start "$SERVICE_NAME"; then
                echo -e "${GREEN}✓ Service started successfully${NC}"
                sleep 2
                if systemctl is-active --quiet "$SERVICE_NAME"; then
                    echo -e "${GREEN}✓ Service is running and active${NC}"
                else
                    echo -e "${YELLOW}⚠ Service started but may need configuration review${NC}"
                fi
            else
                echo -e "${RED}✗ Failed to start service${NC}"
                echo -e "${YELLOW}Please check configuration and start manually: systemctl start $SERVICE_NAME${NC}"
            fi
        fi
    fi
    
    echo ""
}

finalize_installation() {
    echo -e "${BLUE}Finalizing installation...${NC}"
    
    # Enable service for automatic startup
    systemctl enable "$SERVICE_NAME"
    echo -e "${GREEN}✓ Service enabled for automatic startup${NC}"
    echo ""
    
    # Handle service startup
    handle_service_startup
    
    # Show completion message based on installation type
    if [ "${UPDATE_INSTALL:-}" = "true" ]; then
        echo -e "${GREEN}${BOLD}Update completed successfully!${NC}"
        echo ""
        echo -e "${BLUE}Update summary:${NC}"
        echo "• Application files updated"
        echo "• Configuration preserved and backed up to: ${BACKUP_DIR:-/tmp/ispprotect_backup_*}"
        if [ -f "${INSTALL_DIR}/config.inc.php.new" ]; then
            echo "• New configuration template: ${INSTALL_DIR}/config.inc.php.new"
        fi
        echo ""
        if [ "${SERVICE_WAS_RUNNING:-}" = "true" ]; then
            echo -e "${BLUE}Service status:${NC}"
            echo "• Service was automatically restarted"
        else
            echo -e "${BLUE}Next steps:${NC}"
            echo "1. Review new configuration options (if any)"
            echo "2. Start the service: systemctl start $SERVICE_NAME"
        fi
    elif [ "${MIGRATION_INSTALL:-}" = "true" ]; then
        echo -e "${GREEN}${BOLD}Migration completed successfully!${NC}"
        echo ""
        echo -e "${BLUE}Migration summary:${NC}"
        echo "• Migrated from ISPConfig3 BanDaemon"
        echo "• Configuration and data preserved"
        echo "• Updated to systemd service"
        echo ""
        echo -e "${BLUE}Next steps:${NC}"
        echo "1. Review configuration: ${INSTALL_DIR}/config.inc.php"
        echo "2. Remove old installation: rm -rf /opt/ispconfig3_bandaemon"
    else
        echo -e "${GREEN}${BOLD}Installation completed successfully!${NC}"
        echo ""
        echo -e "${BLUE}Next steps:${NC}"
        echo "1. Edit ${INSTALL_DIR}/config.inc.php as needed"
        if ! systemctl is-active --quiet "$SERVICE_NAME"; then
            echo "2. Start the service: systemctl start $SERVICE_NAME"
        fi
    fi
    
    echo ""
    echo -e "${BLUE}Service Management Commands:${NC}"
    echo "• Start service:   systemctl start $SERVICE_NAME"
    echo "• Stop service:    systemctl stop $SERVICE_NAME"
    echo "• Restart service: systemctl restart $SERVICE_NAME"
    echo "• Check status:    systemctl status $SERVICE_NAME"
    echo "• View logs:       journalctl -u $SERVICE_NAME -f"
    echo "• Enable startup:  systemctl enable $SERVICE_NAME"
    echo "• Disable startup: systemctl disable $SERVICE_NAME"
    echo ""
    echo -e "${BLUE}Command-line usage:${NC}"
    echo "• Use 'ispp_bandaemon' command from anywhere (symlinked to /usr/local/bin/)"
    echo "• Use 'ispp_bandaemon --help' for available options"
    echo "• Advanced tools:  ${INSTALL_DIR}/systemd_service_helper.sh"
    echo ""
    echo -e "${YELLOW}Recommended packages:${NC}"
    echo "• php-geoip and geoip-database-contrib for IP geolocation"
    echo "• fail2ban integration (optional)"
    echo ""
}

# Main installation flow
main() {
    print_banner
    check_requirements
    install_files
    create_system_symlink
    create_systemd_service
    handle_existing_installation
    setup_database
    create_configuration
    setup_license
    setup_logrotate
    setup_cron
    finalize_installation
}

# Run main function
main "$@" 