#!/bin/sh

#################################
#      ISPProtect BanDaemon     #
#    Systemd Service Helper     #
#################################

SERVICE_NAME="ispprotect_bandaemon"
INSTALL_DIR="/opt/ispprotect_bandaemon"

# Colors
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
BLUE='\033[0;34m'
NC='\033[0m'

show_usage() {
    echo -e "${BLUE}ISPProtect BanDaemon Service Helper${NC}"
    echo ""
    echo "Usage: $0 {start|stop|restart|status|logs|enable|disable|migrate|health}"
    echo ""
    echo "Commands:"
    echo "  start     - Start the daemon service"
    echo "  stop      - Stop the daemon service" 
    echo "  restart   - Restart the daemon service"
    echo "  status    - Show service status"
    echo "  logs      - Show recent logs (follow mode)"
    echo "  enable    - Enable service to start at boot"
    echo "  disable   - Disable service from starting at boot"
    echo "  migrate   - Migrate from old init.d script to systemd"
    echo "  health    - Show daemon health and statistics"
    echo ""
}

service_start() {
    echo -e "${YELLOW}Starting $SERVICE_NAME...${NC}"
    if systemctl start $SERVICE_NAME; then
        echo -e "${GREEN}✓ Service started successfully${NC}"
        systemctl --no-pager status $SERVICE_NAME
    else
        echo -e "${RED}✗ Failed to start service${NC}"
        return 1
    fi
}

service_stop() {
    echo -e "${YELLOW}Stopping $SERVICE_NAME...${NC}"
    if systemctl stop $SERVICE_NAME; then
        echo -e "${GREEN}✓ Service stopped successfully${NC}"
    else
        echo -e "${RED}✗ Failed to stop service${NC}"
        return 1
    fi
}

service_restart() {
    echo -e "${YELLOW}Restarting $SERVICE_NAME...${NC}"
    if systemctl restart $SERVICE_NAME; then
        echo -e "${GREEN}✓ Service restarted successfully${NC}"
        systemctl --no-pager status $SERVICE_NAME
    else
        echo -e "${RED}✗ Failed to restart service${NC}"
        return 1
    fi
}

service_status() {
    echo -e "${BLUE}Service Status:${NC}"
    systemctl --no-pager status $SERVICE_NAME
    echo ""
    
    if systemctl is-enabled $SERVICE_NAME >/dev/null 2>&1; then
        echo -e "${GREEN}✓ Service is enabled (will start at boot)${NC}"
    else
        echo -e "${YELLOW}⚠ Service is disabled (will not start at boot)${NC}"
    fi
}

service_logs() {
    echo -e "${BLUE}Showing logs for $SERVICE_NAME (Ctrl+C to exit):${NC}"
    echo ""
    journalctl -u $SERVICE_NAME -f --no-pager
}

service_enable() {
    echo -e "${YELLOW}Enabling $SERVICE_NAME to start at boot...${NC}"
    if systemctl enable $SERVICE_NAME; then
        echo -e "${GREEN}✓ Service enabled${NC}"
    else
        echo -e "${RED}✗ Failed to enable service${NC}"
        return 1
    fi
}

service_disable() {
    echo -e "${YELLOW}Disabling $SERVICE_NAME from starting at boot...${NC}"
    if systemctl disable $SERVICE_NAME; then
        echo -e "${GREEN}✓ Service disabled${NC}"
    else
        echo -e "${RED}✗ Failed to disable service${NC}"
        return 1
    fi
}

migrate_from_initd() {
    echo -e "${BLUE}Migrating from init.d to systemd...${NC}"
    echo ""
    
    # Stop old services
    for old_service in ispprotect_bandaemon ispconfig3_bandaemon; do
        if [ -f "/etc/init.d/$old_service" ]; then
            echo -e "${YELLOW}Found old init script: /etc/init.d/$old_service${NC}"
            
            # Stop if running
            if service $old_service status >/dev/null 2>&1; then
                echo -e "${YELLOW}Stopping old service...${NC}"
                service $old_service stop
            fi
            
            # Disable from startup
            if command -v update-rc.d >/dev/null 2>&1; then
                update-rc.d $old_service disable 2>/dev/null || true
            elif command -v chkconfig >/dev/null 2>&1; then
                chkconfig $old_service off 2>/dev/null || true
            fi
            
            # Backup and remove
            echo -e "${YELLOW}Backing up old init script...${NC}"
            mv "/etc/init.d/$old_service" "/etc/init.d/$old_service.backup.$(date +%Y%m%d)"
        fi
    done
    
    # Install systemd service if not present
    if [ ! -f "/etc/systemd/system/$SERVICE_NAME.service" ]; then
        echo -e "${RED}Error: systemd service file not found!${NC}"
        echo "Please run the install.sh script first."
        return 1
    fi
    
    # Enable and start new service
    systemctl daemon-reload
    systemctl enable $SERVICE_NAME
    
    echo -e "${GREEN}✓ Migration completed${NC}"
    echo ""
    echo "You can now use standard systemd commands:"
    echo "  systemctl start/stop/restart/status $SERVICE_NAME"
    echo "  journalctl -u $SERVICE_NAME"
}

service_health() {
    echo -e "${BLUE}ISPProtect BanDaemon Health Check${NC}"
    echo "=================================="
    echo ""
    
    # Service status
    if systemctl is-active $SERVICE_NAME >/dev/null 2>&1; then
        echo -e "${GREEN}✓ Service is running${NC}"
    else
        echo -e "${RED}✗ Service is not running${NC}"
    fi
    
    # PID file check
    if [ -f "/run/ispprotect_bandaemon.pid" ]; then
        PID=$(cat /run/ispprotect_bandaemon.pid)
        if kill -0 $PID 2>/dev/null; then
            echo -e "${GREEN}✓ Process is alive (PID: $PID)${NC}"
        else
            echo -e "${RED}✗ PID file exists but process is dead${NC}"
        fi
    else
        echo -e "${YELLOW}⚠ PID file not found${NC}"
    fi
    
    # Configuration check
    if [ -f "$INSTALL_DIR/config.inc.php" ]; then
        echo -e "${GREEN}✓ Configuration file exists${NC}"
    else
        echo -e "${RED}✗ Configuration file missing${NC}"
    fi
    
    # License check
    if [ -f "$INSTALL_DIR/license.key" ]; then
        echo -e "${GREEN}✓ License file exists${NC}"
    else
        echo -e "${YELLOW}⚠ License file missing${NC}"
    fi
    
    # Log file check
    if [ -f "/var/log/ispprotect_bandaemon.log" ]; then
        # Try different stat commands for portability (BSD vs GNU)
        if command -v stat >/dev/null 2>&1; then
            LOG_SIZE=$(stat -c%s "/var/log/ispprotect_bandaemon.log" 2>/dev/null || stat -f%z "/var/log/ispprotect_bandaemon.log" 2>/dev/null || echo "unknown")
        else
            LOG_SIZE="unknown"
        fi
        echo -e "${GREEN}✓ Log file exists (${LOG_SIZE} bytes)${NC}"
        
        # Show last few log entries
        echo ""
        echo -e "${BLUE}Recent log entries:${NC}"
        tail -5 /var/log/ispprotect_bandaemon.log 2>/dev/null || echo "No recent entries"
    else
        echo -e "${YELLOW}⚠ Log file not found${NC}"
    fi
    
    # Database connection test (if available)
    if [ -f "$INSTALL_DIR/ispp_bd_helper.php" ]; then
        echo ""
        echo -e "${BLUE}Testing database connection...${NC}"
        if php "$INSTALL_DIR/ispp_bd_helper.php" --test-db 2>/dev/null; then
            echo -e "${GREEN}✓ Database connection OK${NC}"
        else
            echo -e "${YELLOW}⚠ Could not test database connection${NC}"
        fi
    fi
}

# Main script logic
case "$1" in
    start)
        service_start
        ;;
    stop)
        service_stop
        ;;
    restart)
        service_restart
        ;;
    status)
        service_status
        ;;
    logs)
        service_logs
        ;;
    enable)
        service_enable
        ;;
    disable)
        service_disable
        ;;
    migrate)
        migrate_from_initd
        ;;
    health)
        service_health
        ;;
    *)
        show_usage
        exit 1
        ;;
esac

exit 0 